<?php

namespace App\Helpers;

class PaymentClient
{

    public static function getPaymentLink($price, $orderId, $userId){
        if($price == 0)
            return null;
        /*************ABOUT this API************
        ****************************************
        1. This is 'create order request' api, that generates the payment link. Once this API is triggered with the required datapoints then our systems will return a payment link.
        2. This API strictly uses cURL and REST API based posting, no HTTP method allowed.
        *****************************************
        ***************************************/ 
        $cartid_1= $orderId.rand(1,1000000);  // This generates Random cart id value, please replace this with your Order id generating code.

        $storeid =env('TLR_STORE_ID');  //Store ID here
        $authkey = env('TLR_AUTH_KEY'); //Authentication key can be found here - Telr admin-> Integrations -> Payment page V2 -> configuration -> Authentication key


        //Redirection to merchant website post transaction completion, note there will be no data posted from Telr to these URL's, instead its just a redirection.
        $authurl = env('APP_URL').'/w/authorizedPayment?Txnorder='.$cartid_1;
        $decl = env('APP_URL').'/w/declinedPayment?Txnorder='.$cartid_1;
        $cncl = env('APP_URL').'/w/cancelPayment?Txnorder='.$cartid_1;

        //define TEST mode or LIVE mode (1 is TEST mode, 0 is LIVE mode)
        $environment = '1';  // Either 1 Or 0

        /**************/
        // Store Card
        // 1) Store card unique id for every customer, ensure this is not passed for other customer else they can see other customers saved card.
        // 2) Can be maximum 20 alphanumeric value
        // 3) Store card feature only works if customer chooses to Save their card data post completing Successful transaction.
        

        //Sample Customer details, please note below details mandatory and needs to be captured on merchant end and needs to be geniune details of the customer. Also if any of these details are not passed in the payment request then the same will be asked on the payment page.  
        /*  'bill_fname' =>'Rahul',
            'bill_sname' =>'U',
            'bill_addr1' =>'Test address Test tested t',
            'bill_addr2' =>'Address 2345 test ',
            'bill_city' =>'Dubai',
            'bill_country' =>'SA',
            'bill_phone' =>'8888888888',
            'bill_email' =>'rahul.upadhayay@telr.ccom',
            'bill_zip' =>'11111',
            'bill_region' =>'Arab Emirates',  
        */

        
        $custref = $userId;  //Store card unique id for each customer : (prefix+phone number)


        $params = array(
            'ivp_method'  => 'create',
            'ivp_store'   => $storeid,
            'ivp_authkey' => $authkey,
            'ivp_cart'    => $cartid_1,
            'bill_custref' => $custref,
            
            // 'bill_fname' =>'Rahul',
            // 'bill_sname' =>'U',
            'bill_addr1' =>'Test address Test tested t',// random
            'bill_addr2' =>'Address 2345 test ',// random or empty
            'bill_city' =>'Dubai',// random
            'bill_country' =>'SA',
            'bill_phone' =>'8888888888',// random
            // 'bill_email' =>'rahul.upadhayay@telr.ccom',// random
            'bill_zip' =>'11111',// random
            'bill_region' =>'Arab Emirates',

            'ivp_test'    => $environment,
            'ivp_amount'  => strval($price),
            'ivp_lang' => 'en',
            'ivp_framed' => '0',
            'ivp_currency'=> 'AED',
            'ivp_desc'    => 'Product Description',
            'return_auth' => $authurl,
            'return_can'  => $decl,
            'return_decl' => $cncl,
            // "splitpayment": { "id": <id_value> }
        );
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, env('TLR_URL')."/order.json");
        curl_setopt($ch, CURLOPT_POST, count($params));
        curl_setopt($ch, CURLOPT_POSTFIELDS,$params);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));
        $results = curl_exec($ch);
        curl_close($ch);

        $results = json_decode($results);
        return $results;
    }

    public static function check($orderRef){
        $storeid =env('TLR_STORE_ID');  //Store ID here
        $authkey = env('TLR_AUTH_KEY');
        $order = array("ref"=>$orderRef);
        $client = new \GuzzleHttp\Client();

        $response = $client->request('POST', 'https://secure.telr.com/gateway/order.json', [
          'body' => '{"method":"check","store":'.$storeid.',"authkey":"'.$authkey.'","order":{"ref":"'.$orderRef.'"}}',
          'headers' => [
            'Content-Type' => 'application/json',
            'accept' => 'application/json',
          ],
        ]);
        $result = json_decode(str($response->getBody()));
        return $result->order->status->text;

        // $params = array(
        //     'ivp_method'  => 'check',
        //     'ivp_store'   => $storeid,
        //     'ivp_authkey' => $authkey,
        //     'ivp_order' => $order,
        // );
        // $ch = curl_init();
        // curl_setopt($ch, CURLOPT_URL, env('TLR_URL')."/order.json");
        // curl_setopt($ch, CURLOPT_POST, count($params));
        // curl_setopt($ch, CURLOPT_POSTFIELDS,$params);
        // curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        // curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));
        // $results = curl_exec($ch);
        // curl_close($ch);

        // $results = json_decode($results);
        // return $results;
    }

}