<?php

namespace App\Http\Controllers;

use App\Models\Place;
use App\Models\User;
use App\Models\Ticket;
use App\Models\Ads;
use App\Models\Assignment;
use App\Models\Promotion;
use App\Models\CarDamage;
use Illuminate\Http\Request;
use App\Rules\ArrayValuesInList;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Hash;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Broadcasting\BroadcastException;
use Twilio\Rest\Client;
use Illuminate\Database\Eloquent\Builder;
use Exception;
use App\Helpers\SMSClient;
use App\Helpers\WhatsappClient;
use App\Helpers\PaymentClient;
use App\Helpers\EmailClient;
use App\Helpers\PusherClient;
use App\Enums\CarStatus;
use AshAllenDesign\ShortURL\Facades\ShortURL;
use Illuminate\Support\Carbon;
use App\Helpers\FirebaseClient;

class AssignmentController extends Controller
{
    public function addQRAssignment(Request $request){
        $validator = Validator::make($request->all(), [
            'ticket_uuid' => ['required', 'string', 'size:36', 'exists:tickets,id'],
            'firstname' => ['max:255'],
            'lastname' => ['max:255'],
            'is_guest' => Rule::in([0, 1]),
            'room' => ['string', 'max:8'],
            'carwash' => Rule::in([0, 1]),
            'fuel' => Rule::in([0, 1]),
            'checkout_date' => 'date',
            'car_plate' => ['max:50'],
            'status' => ['string', Rule::in(['parking', 'request_and_paid', 'request_not_paid', 'outdoor', 'released'])],
            'car_make' => ['max:50'],
            'car_color' => ['max:255'],
            'level' => ['max:70'],
            'slot' => ['max:70'],
            'hanger' => ['max:70'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());

        $data = $validator->validated();
        try{
            $ticket = Ticket::find($data['ticket_uuid']);
            if($ticket->status){
                return response()->error(6000, "this ticket is accupied!", []);
            }

            DB::beginTransaction();
            $user = auth()->user();
            $assignment = Assignment::create([
                'ticket_uuid' => $data['ticket_uuid'],
                'user_uuid' => $user->id,
            ]);
            $name = "";
            if($request->has('firstname')){
                if($data['firstname'] != "")
                    $assignment->firstname = $data['firstname'];
                $name .= $data['firstname']." ";
            }
            if($request->has('lastname')){
                if($data['lastname'] != "")
                    $assignment->lastname = $data['lastname'];
                $name .= $data['lastname'];
            }
            if($request->has('email')){
                if($data['email'] != "")
                    $assignment->email = $data['email'];
            }
            if($request->has('phone')){
                if($data['phone'] != "")
                    $assignment->phone = $data['phone'];
            }
            if($request->has('country_code')){
                if($data['country_code'] != "")
                    $assignment->country_code = $data['country_code'];
            }
            if($request->has('send_sms')){
                if($data['send_sms'] != "")
                    $assignment->send_sms = $data['send_sms'];
            }
            if($request->has('is_guest')){
                $assignment->is_guest = $data['is_guest'];
            }
            if($request->has('room')){
                $assignment->room = $data['room'];
            }
            if($request->has('carwash')){
                $assignment->carwash = $data['carwash'];
            }
            if($request->has('fuel')){
                $assignment->fuel = $data['fuel'];
            }
            if($request->has('checkout_date')){
                $assignment->checkout_date = $data['checkout_date'];
            }
            if($request->has('car_plate')){
                $assignment->car_plate = $data['car_plate'];
            }
            if($request->has('car_color')){
                $assignment->car_color = $data['car_color'];
            }
            if($request->has('car_make')){
                $assignment->car_make = $data['car_make'];
            }
            if($request->has('level')){
                $assignment->level = $data['level'];
            }
            if($request->has('slot')){
                $assignment->slot = $data['slot'];
            }
            if($request->has('hanger')){
                $assignment->hanger = $data['hanger'];
            }
            $ticket->status = 1;
            $assignment->status = CarStatus::PARKING;
            $assignment->save();
            $ticket->save();
	        $builder = new \AshAllenDesign\ShortURL\Classes\Builder();
            $signed_url = URL::signedRoute('userInfo', ['assignment' => $assignment->id], absolute: false);
            //$shortURL = $signed_url;
            $shortURLObject = $builder->destinationUrl(env('APP_URL').$signed_url)->redirectStatusCode(302)->make();
            $shortURL = $shortURLObject->default_short_url;

            DB::commit();
            return response()->standard("Assignment was added successfully.",[
                "id" => $assignment->id,
                "assignment" => $assignment,
                "url" => $shortURL,
                "phone" => $assignment->country_code.$assignment->phone,
            ]);
        }catch(QueryException $e){
            DB::rollBack();
            return response()->error(1001, "SQL Error", $e);
        }catch(Exception $e){
            DB::rollBack();
            return response()->error(1002, "Some Error accured", $e->getMessage());
        }        
    }
    /**
     * Add an assignment.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function addAssignment(Request $request){
        $validator = Validator::make($request->all(), [
            'ticket_uuid' => ['required', 'string', 'size:36', 'exists:tickets,id'],
            'phone' => ['required_without:email'],
            'country_code' => ['required_without:email'],
            'email' => ['required_without:phone'],
            'send_sms' => ['required'],
            'firstname' => ['required_without:lastname','max:255'],
            'lastname' => ['required_without:firstname', 'max:255'],
            'is_guest' => Rule::in([0, 1]),
            'room' => ['string', 'max:8'],
            'carwash' => Rule::in([0, 1]),
            'fuel' => Rule::in([0, 1]),
            'checkout_date' => 'date',
            'car_plate' => ['max:50'],
            'status' => ['string', Rule::in(['parking', 'request_and_paid', 'request_not_paid', 'outdoor', 'released'])],
            'car_make' => ['max:50'],
            'car_color' => ['max:255'],
            'level' => ['max:70'],
            'slot' => ['max:70'],
            'hanger' => ['max:70'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());

        $data = $validator->validated();
        try{
            $ticket = Ticket::find($data['ticket_uuid']);
            if($ticket->status){
                return response()->error(6000, "this ticket is accupied!", []);
            }

            DB::beginTransaction();
            $user = auth()->user();
            $assignment = Assignment::create([
                'ticket_uuid' => $data['ticket_uuid'],
                'user_uuid' => $user->id,
            ]);
            $name = "";
            if($request->has('firstname')){
                if($data['firstname'] != "")
                    $assignment->firstname = $data['firstname'];
                $name .= $data['firstname']." ";
            }
            if($request->has('lastname')){
                if($data['lastname'] != "")
                    $assignment->lastname = $data['lastname'];
                $name .= $data['lastname'];
            }
            if($request->has('email')){
                if($data['email'] != "")
                    $assignment->email = $data['email'];
            }
            if($request->has('phone')){
                if($data['phone'] != "")
                    $assignment->phone = $data['phone'];
            }
            if($request->has('country_code')){
                if($data['country_code'] != "")
                    $assignment->country_code = $data['country_code'];
            }
            if($request->has('send_sms')){
                if($data['send_sms'] != "")
                    $assignment->send_sms = $data['send_sms'];
            }
            if($request->has('is_guest')){
                $assignment->is_guest = $data['is_guest'];
            }
            if($request->has('room')){
                $assignment->room = $data['room'];
            }
            if($request->has('carwash')){
                $assignment->carwash = $data['carwash'];
            }
            if($request->has('fuel')){
                $assignment->fuel = $data['fuel'];
            }
            if($request->has('checkout_date')){
                $assignment->checkout_date = $data['checkout_date'];
            }
            if($request->has('car_plate')){
                $assignment->car_plate = $data['car_plate'];
            }
            if($request->has('car_color')){
                $assignment->car_color = $data['car_color'];
            }
            if($request->has('car_make')){
                $assignment->car_make = $data['car_make'];
            }
            if($request->has('level')){
                $assignment->level = $data['level'];
            }
            if($request->has('slot')){
                $assignment->slot = $data['slot'];
            }
            if($request->has('hanger')){
                $assignment->hanger = $data['hanger'];
            }
            $ticket->status = 1;
            $assignment->status = CarStatus::PARKING;
            $assignment->save();
            $ticket->save();
	        $builder = new \AshAllenDesign\ShortURL\Classes\Builder();
            $signed_url = URL::signedRoute('requestCar', ['assignment' => $assignment->id], absolute: false);
            //$shortURL = $signed_url;
            $shortURLObject = $builder->destinationUrl(env('APP_URL').$signed_url)->make();
            $shortURL = $shortURLObject->default_short_url;

            $messageTemplates = config('app-messages');
            if($data['phone'] != ""){
                if($data["send_sms"] == 1){
                    $message = str_replace(
                        ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                        [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                        $messageTemplates["sms"]
                    );
                    $smsResult = SMSClient::send($assignment->country_code . $assignment->phone, $message);
                    $assignment->sms_id = $smsResult->data->id;
                    $assignment->save();
                }
                else{
                    $message = str_replace(
                        ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                        [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                        $messageTemplates["whatsapp"]
                    );
                    WhatsappClient::send(substr($assignment->country_code . $assignment->phone, 1, strlen($assignment->country_code . $assignment->phone)), $message);
                    $assignment->sms_id =null;
                    $assignment->save();
                }
            }


            if($data['email'] != ""){
                $message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["email"]
                );
                EmailClient::send($data['email'], $name, $assignment->place.":Valet Ticket", $message);
            }

            DB::commit();
            return response()->standard("Assignment was added successfully.",[
                "id" => $assignment->id,
                "assignment" => $assignment,
                "url" => $shortURL,
                "phone" => $assignment->country_code.$assignment->phone,
            ]);
        }catch(QueryException $e){
            DB::rollBack();
            return response()->error(1001, "SQL Error", $e);
        }catch(Exception $e){
            DB::rollBack();
            return response()->error(1002, "Some Error accured", $e->getMessage());
        }
    }

    /**
     * Edit an assignment.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function editAssignment(Request $request){
        $validator = Validator::make($request->all(), [
            'assignment_uuid' => ['required', 'string', 'size:36', 'exists:assignments,id'],
            'phone' => ['max:255'],
            'country_code' => ['required_without:email'],
            'email' => ['max:255'],
            'send_sms' => ['required'],
            'firstname' => ['max:255'],
            'lastname' => ['max:255'],
            'is_guest' => Rule::in([0, 1]),
            'room' => ['string', 'max:8'],
            'carwash' => Rule::in([0, 1]),
            'fuel' => Rule::in([0, 1]),
            'checkout_date' => 'date',
            'car_plate' => ['max:50'],
            'car_make' => ['max:50'],
            'car_color' => ['max:255'],
            'level' => ['max:70'],
            'slot' => ['max:70'],
            'hanger' => ['max:70'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());

        $data = $validator->validated();
        try{
            $assignment = Assignment::find($data['assignment_uuid']);
            if($assignment == null){
                return response()->standard("Assignment not found", []);
            }

            DB::beginTransaction();
            $user = auth()->user();
            if($user->id != $assignment->user_uuid){
                return response()->unauthorized();
            }

            $emailChanged = false;
            $phoneChanged = false;
            $sendingMethodChanged = false;

            $name = "";
            if (($assignment->firstname != null || $assignment->lastname != null) && ($request->has('firstname') == false && $request->has('lastname') == false)) {
                $validator->errors()->add('empty_entry', 'You cannot return an empty entry when your field is full! Please check firstname and lastname.');
                return response()->validation($validator->errors());            
            }            
            if(($assignment->firstname != null || $assignment->lastname != null) && (empty($data['firstname']) && empty($data['lastname']))){
                $validator->errors()->add('empty_entry', 'You cannot return an empty entry when your field is full! Please check firstname and lastname.');
                return response()->validation($validator->errors());   
            }
            if(($assignment->email != null || $assignment->phone != null) && ($request->has('email')== false && $request->has('phone')== false)){
                $validator->errors()->add('empty_entry', 'You cannot return an empty entry when your field is full! check phone / email ');
                return response()->validation($validator->errors());   
            }
            if(($assignment->email != null || $assignment->phone != null) && (empty($data['email']) && empty($data['phone']))){
                $validator->errors()->add('empty_entry', 'You cannot return an empty entry when your field is full! check phone / email ');
                return response()->validation($validator->errors());   
            }
            if($request->has('firstname')){
                    $assignment->firstname = $data['firstname'];
                $name .= $data['firstname']." ";
            }
            if($request->has('lastname')){
                    $assignment->lastname = $data['lastname'];
                $name .= $data['lastname'];
            }

            if($request->has('email')){
                    $assignment->email = $data['email'];
            }

            if($request->has('phone')){
                    $assignment->phone = $data['phone'];
            }

            if($request->has('country_code')){
                if($data['country_code'] != "")
                    $assignment->country_code = $data['country_code'];
            }
            if($assignment->send_sms != $data['send_sms']){
                $sendingMethodChanged = true;
            }
            if($request->has('send_sms')){
                if($data['send_sms'] != "")
                    $assignment->send_sms = $data['send_sms'];
            }
            if($request->has('is_guest')){
                $assignment->is_guest = $data['is_guest'];
            }
            if($request->has('room')){
                $assignment->room = $data['room'];
            }
            if($request->has('carwash')){
                $assignment->carwash = $data['carwash'];
            }
            if($request->has('fuel')){
                $assignment->fuel = $data['fuel'];
            }
            if($request->has('checkout_date')){
                $assignment->checkout_date = $data['checkout_date'];
            }
            if($request->has('car_plate')){
                $assignment->car_plate = $data['car_plate'];
            }
            if($request->has('car_color')){
                $assignment->car_color = $data['car_color'];
            }
            if($request->has('car_make')){
                $assignment->car_make = $data['car_make'];
            }
            if($request->has('level')){
                $assignment->level = $data['level'];
            }
            if($request->has('slot')){
                $assignment->slot = $data['slot'];
            }
            if($request->has('hanger')){
                $assignment->hanger = $data['hanger'];
            }

            $assignment->save();

            $shortURL = DB::table('short_urls')->where('destination_url', 'LIKE', '%'.$assignment->id.'%')->get('default_short_url')[0]->default_short_url;


            $messageTemplates = config('app-messages');
            if($phoneChanged || $sendingMethodChanged){
                if($data['phone'] != ""){
                    if($data["send_sms"] == 1){
                        $message = str_replace(
                            ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                            [$assignment->ticket->label, $assignment->ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                            $messageTemplates["sms"]
                        );
                        $smsResult = SMSClient::send($assignment->country_code . $assignment->phone, $message);
                        $assignment->sms_id = $smsResult->data->id;
                    }
                    else{
                        $message = str_replace(
                            ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                            [$assignment->ticket->label, $assignment->ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                            $messageTemplates["whatsapp"]
                        );
                        WhatsappClient::send(substr($assignment->country_code . $assignment->phone, 1, strlen($assignment->country_code . $assignment->phone)), $message);
                        $assignment->sms_id =null;
                    }
                }
            }

            if($emailChanged){
                $message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$assignment->ticket->label, $assignment->ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["email"]
                );
                EmailClient::send($data['email'], $name, $assignment->place.":Valet Ticket", $message);
            }
            DB::commit();
            return response()->standard("Assignment was updated successfully.",[
                "id" => $assignment->id,
                "assignment" => $assignment,
                "url" => $shortURL,
                "phone" => $assignment->country_code.$assignment->phone,
            ]);
        }catch(QueryException $e){
            DB::rollBack();
            return response()->error(1001, "SQL Error", $e);
        }catch(Exception $e){
            DB::rollBack();
            return response()->error(1002, "Some Error accured", $e->getMessage());
        }
    }

    /**
     * Get a specific assignment.
     *
     * @param string $assignment_uuid The assignment UUID.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function getAssignment(string $assignment_uuid){
        $validator = Validator::make(["assignment_uuid" => $assignment_uuid], [
            'assignment_uuid' => ['required', 'string', 'size:36', 'exists:assignments,id'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());
        try{
            $data = $validator->validated();
            $user = auth()->user();
            $assignment = Assignment::find($data['assignment_uuid']);
            if($assignment == null)
                return response()->standard("assignment not found", []);
            if(auth()->payload()->get('default_place') == $assignment->ticket->place->id){
                return response()->standard("", ["assignment" => $assignment, "is_valleydesk" => $user->is_valleydesk]);
            }else{
                return response()->unauthorized();
            }
        }catch(QueryException $e){
            return response()->error(1001, "SQL Error", $e);
        }catch(Exception $e){
            return response()->error(1002, "Some Error accured", $e->getMessage());
        }
    }

    /**
     * Get a list of assignments.
     *
     * @param int $page The page number.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function getAssignmentList(int $page = 1){
        $limit = 100;
        $offset = $limit*($page - 1);
        $user = auth()->user();
        if($user->places->find(auth()->payload()->get('default_place'))->pivot->permissions->contains("MANAGER")
            ||
            $user->places->find(auth()->payload()->get('default_place'))->pivot->permissions->contains("VALET_DESK")
            ||
            $user->places->find(auth()->payload()->get('default_place'))->pivot->permissions->contains("RECEPTION")){
                $assignments = Assignment::with(['ticket'])
                    ->whereIn('ticket_uuid', $user->places->find(auth()->payload()->get('default_place'))->tickets->pluck('id'))
                    ->where('status','!=','released')
                    ->orderBy('updated_at', 'desc')
                    ->offset($offset)->limit($limit)
                    ->get();

                return response()
                    ->standard("", [
                        "page" => $page,
                        "assignments" => $assignments,
                        "is_valleydesk" => $user->places->find(auth()->payload()->get('default_place'))->pivot->permissions->contains("VALET_DESK")
                    ]);
        }elseif ($user->places->find(auth()->payload()->get('default_place'))->pivot->permissions->contains("VALET")) {
            $assignments = Assignment::with(['ticket'])
                                    ->whereIn('ticket_uuid', $user->places->find(auth()->payload()->get('default_place'))->tickets->pluck('id'))
                                    ->where('status','!=','released')
                                    ->where(function (Builder $query){
                                        $query->where(function (Builder $query){
                                            $query->where('user_uuid', auth()->user()->id)
                                                  ->where('assigned_to', null);
                                        })
                                              ->orWhere('assigned_to', auth()->user()->id)
                                              ->orWhere(function (Builder $query){
                                                    $query->where('assigned_to', null)
                                                          ->whereIn('status', ['requested_and_paid', 'requested_and_not_paid']);
                                              });
                                    })
                                    ->orderBy('assigned_to')
                                    // ->orderByRaw('FIELD(status,requested_and_paid,requested_and_not_paid)')
                                    ->orderBy('updated_at', 'desc')
                                    ->offset($offset)->limit($limit)
                                    ->get();
            return response()
                ->standard("", [
                    "page" => $page,
                    "assignments" => collect($assignments)->sortByDesc('updated_at')->values()->all(),
                    "is_valleydesk" => $user->places->find(auth()->payload()->get('default_place'))->pivot->permissions->contains("VALET_DESK")]);
        }else{
            return response()->unauthorized();
        }
    }

    /**
     * Delete an assignment.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function deleteAssignment(Request $request){
        $validator = Validator::make($request->all(), [
            'id' => ['required', 'string', 'size:36', 'exists:assignments,id'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());
        $data = $validator->validated();
        $assignment = Assignment::find($data["id"]);
        if(auth()->payload()->get('default_place') == $assignment->ticket->place->id){
            $assignment->delete();
            return response()->standard("Assignment delete successfully", []);
        }else{
            return response()->unauthorized();
        }
    }


    /**
     * Retrieves the list of colors.
     *
     * @return \Illuminate\Http\JsonResponse The list of colors as a JSON response.
     */
    public function selectColor(){
        $colors = config('color');
        $colorList = [];
        foreach($colors as $name => $code){
            array_push($colorList, [
                "color_name" => $name,
                "color_code" => $code
            ]);
        }
        return response()->standard("", $colorList);
    }

    /**
     * Retrieves the list of car brands.
     *
     * @return \Illuminate\Http\JsonResponse The JSON response containing the list of car brands.
     */
    public function selectCarBrand(){
        $cars = config('car-brands');
        return response()->standard("", $cars);
    }

    /**
     * Check a ticket.
     *
     * @param string $label The label of the ticket.
     * @throws Some_Exception_Class Description of exception
     * @return Some_Return_Value
     */
    public function checkTicket(string $label){
        $validator = Validator::make(['label' => $label], [
            'label' => ['required', 'string', 'min:1']
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());

        $data = $validator->validated();
        $ticket = Ticket::where('place_uuid', auth()->payload()->get('default_place'))->where('label', $label)->get()->last();
        if($ticket != null){
            $id = $ticket->id;
            $status = $ticket->status;
            if($status == 1)
                $message = "Ticket accupied";
            else
                $message = "Ticket is free";
        }
        else{
            return response()->error(404, "Ticket is unavailable", []);
        }

        return response()->standard($message, ['ticket_uuid' => $id, 'status' => $status, 'assignment' => $ticket->assignments->last()]);
    }

     /**
     * Accepts a request and assigns an assignment to the authenticated user.
     *
     * @param Request $request The request object containing the assignment UUID.
     * @throws Exception An exception is thrown if there is an error assigning the assignment.
     * @return mixed Returns the response object with the assigned assignment or an error message.
     */
    public function acceptRequest(Request $request){
        $validator = Validator::make($request->all(), [
            'assignment_uuid' => ['required', 'exists:assignments,id']
        ]);
        if($validator->fails()){
            return response()->validation($validator->errors());
        }
        $user =  auth()->user();
        $data = $validator->validated();
        $assignment = Assignment::find($data['assignment_uuid']);
        if(
            $assignment->ticket->place->id == auth()->payload()->get('default_place') &&
            is_null($assignment->assigned_to) &&
            !is_null($assignment->requested_at)
        ){
            try{
                $assignment->assigned_to = $user->id;
                $assignment->assigned_at = date('Y-m-d H:i:s', strtotime('now'));
                $assignment->save();
                PusherClient::send(
                    'car_'.auth()->payload()->get('default_place'),
                    'new_car',
                    $assignment
                );
                $notif_config = config('notification-message-config');
                FirebaseClient::send(
                    'car_'.auth()->payload()->get('default_place'),
                    $notif_config['title'],
                    $notif_config['body']
                );
                return response()->standard('Assignment assigned to you.', $assignment);
            }catch(Exception $e){
                return response()->error(400, "", $e);
            }

        }else{
            if($assignment->ticket->place->id != auth()->payload()->get('default_place'))
                return response()->error(401, "Not authorized", null);
            if(!is_null($assignment->assigned_to))
                return response()->error(400, "Assigned to another one", []);
            if(is_null($assignment->requested_at))
                return response()->error(400, "Not requested", null);
            else{
                return response()->error(400, "---", [$assignment->ticket->place->id == auth()->payload()->get('default_place') ,
                is_null($assignment->assigned_to) ,
                !is_null($assignment->requested_at)]);
            }
        }
    }

    /**
     * Validates the request and prepares an assignment for release.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class If the validation fails.
     * @return Response The response object.
     */
    public function readyAssignment(Request $request){
        $validator = Validator::make($request->all(), [
            'assignment_uuid' => ['required', 'exists:assignments,id']
        ]);
        if($validator->fails()){
            return response()->validation($validator->errors());
        }
        $user =  auth()->user();
        $data = $validator->validated();
        $assignment = Assignment::find($data['assignment_uuid']);
        if(
            $assignment->ticket->place->id == auth()->payload()->get('default_place') &&
            $assignment->assigned_to == $user->id &&
            !is_null($assignment->requested_at)
        ){
            if($assignment->status == CarStatus::REQUESTED_PAID)
                $assignment->status = CarStatus::READY_PAID;
            if($assignment->status == CarStatus::REQUESTED_NOT_PAID)
                $assignment->status = CarStatus::READY_NOT_PAID;
            PusherClient::send(
                'car_'.auth()->payload()->get('default_place'),
                'new_car',
                $assignment
            );
            $notif_config = config('notification-message-config');
                FirebaseClient::send(
                    'car_'.auth()->payload()->get('default_place'),
                    $notif_config['title'],
                    $notif_config['body']
                );
            $assignment->save();
            return response()->standard('Assignment released.', $assignment);
        }
    }

    /**
     * Releases an assignment.
     *
     * @param Request $request The request object containing the assignment UUID.
     * @throws Some_Exception_Class If the assignment UUID is missing or invalid.
     * @return Response The response object containing the updated assignment or a validation error.
     */
    public function releaseAssignment(Request $request){
        $validator = Validator::make($request->all(), [
            'assignment_uuid' => ['required', 'exists:assignments,id']
        ]);
        if($validator->fails()){
            return response()->validation($validator->errors());
        }
        $user =  auth()->user();
        $data = $validator->validated();
        try{
            $assignment = assignment::find($data['assignment_uuid']);
            if(
                $assignment->ticket->place->id == auth()->payload()->get('default_place') &&
                $assignment->assigned_to == $user->id &&
                !is_null($assignment->requested_at)
            ){
                DB::beginTransaction();
                $bill = CustomerController::bill($assignment);
                $assignment->charge = $bill['price'];
                if($assignment->status == CarStatus::READY_NOT_PAID){
                    $assignment->cash_reciever =  $user->id;
                    $assignment->paid = 1;
                }
                $assignment->status = CarStatus::RELEASED;
                $assignment->save();
                $assignment->ticket->status = 0;
                $assignment->ticket->save();
                DB::table('short_urls')->where('destination_url','like','%'.$assignment->id.'%')->update(['destination_url' => env('APP_URL')]);
                DB::commit();
                //update(['deactivated_at' => date("Y-m-d H:i:s", strtotime('now'))]);
                PusherClient::send(
                    'car_'.auth()->payload()->get('default_place'),
                    'new_car',
                    $assignment
                );
                $notif_config = config('notification-message-config');
                FirebaseClient::send(
                    'car_'.auth()->payload()->get('default_place'),
                    $notif_config['title'],
                    $notif_config['body']
                );
                return response()->standard('Assignment released.', $assignment);
            }else{
                return response()->unauthorized();
            }
        }catch(Exception $e){
            return response()->error(1001, "", $e->getMessage());
        }
    }

    /**
     * Resends a message.
     *
     * @param Request $request The request object.
     * @throws QueryException SQL error.
     * @throws Exception Some error occurred.
     * @return Response The response object.
     */
    public function resendMessage(Request $request){
        $validator = Validator::make($request->all(), [
            'assignment_uuid' => ['required', 'string', 'size:36', 'exists:assignments,id'],
            'type' => ['string', 'max:120'],
        ]);

        if($validator->fails())
            return response()->validation($validator->errors());

        $data = $validator->validated();
        try{
            $assignment = Assignment::find($data['assignment_uuid']);
            if($assignment == null){
                return response()->standard("assignment not found", []);
            }

            DB::beginTransaction();
            $user = auth()->user();
            if($user->id != $assignment->user_uuid){
                return response()->unauthorized();
            }

            $ticket = Ticket::find($assignment->ticket->id);

            $shortURL = DB::table('short_urls')->where('destination_url', 'LIKE', '%'.$assignment->id.'%')->get('default_short_url')[0]->default_short_url;

            if($data['type'] == 'resend_message'){
                $messageTemplates = config('app-messages');
                $sms_message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["sms"]
                );
                $whatsapp_message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["whatsapp"]
                );
                $email_message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["email"]
                );
            }elseif($data['type'] == 'key_forget'){
                $messageTemplates = config('keyforget-message');
                $sms_message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$ticket->label, $ticket->place->name,  env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["sms"]
                );
                $whatsapp_message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$ticket->label, $ticket->place->name,  env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["whatsapp"]
                );
                $email_message = str_replace(
                    ['{TICKET_LABEL}', '{HOTEL_NAME}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                    [$ticket->label, $ticket->place->name,  env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                    $messageTemplates["email"]
                );
            }
            
            if($assignment->phone != ""){
                if($assignment->send_sms == 1){
                    $smsResult = SMSClient::send($assignment->country_code . $assignment->phone, $sms_message);
                    $assignment->sms_id = $smsResult->data->id;
                    $assignment->save();
                }
                else{
                    WhatsappClient::send(substr($assignment->country_code . $assignment->phone, 1, strlen($assignment->country_code . $assignment->phone)), $whatsapp_message);
                    $assignment->sms_id =null;
                    $assignment->save();
                }
            }
            if($assignment->email != ""){
                EmailClient::send(
                    $assignment->email,
                    $assignment->firstname." ".$assignment->lastname,
                    $assignment->place.":Valet Ticket",
                    $email_message
                );
            }

            DB::commit();

            return response()->standard("Message sent.",[
                "id" => $assignment->id,
                "assignment" => $assignment,
                "url" => $shortURL,
                "phone" => $assignment->country_code.$assignment->phone,
            ]);

        }catch(QueryException $e){
            DB::rollBack();
            return response()->error(1001, "SQL Error", $e);
        }catch(Exception $e){
            DB::rollBack();
            return response()->error(1002, "Some Error accured", $e->getMessage());
        }
    }
}
