<?php

namespace App\Http\Controllers;

use App\Models\Place;
use App\Models\User;
use App\Models\Ticket;
use App\Models\Ads;
use App\Models\Assignment;
use App\Models\Promotion;
use App\Models\CarDamage;
use App\Models\PlaceAds;
use Illuminate\Http\Request;
use App\Rules\ArrayValuesInList;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Hash;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Broadcasting\BroadcastException;
use Twilio\Rest\Client;
use Illuminate\Database\Eloquent\Builder;
use Exception;
use App\Helpers\SMSClient;
use App\Helpers\PaymentClient;
use App\Helpers\EmailClient;
use App\Helpers\PusherClient;
use App\Helpers\FirebaseClient;
use App\Enums\CarStatus;
use AshAllenDesign\ShortURL\Facades\ShortURL;
use Illuminate\Support\Carbon;
use App\Helpers\WhatsappClient;

class CustomerController extends Controller
{


    public function getAds($assignment){
        try{
        $ads = $assignment->ticket->place->advertisements
                            ->where('expiration', '>=', Carbon::now())
                            ->where('pivot.is_active', 1)->random();
        return $ads;
        }catch(Exception $e){
            return response()->error(401, 'Something went wrong!', []);
        }
    }

     /**
     * Request a car.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function requestCar(Request $request){
        $assignment = Assignment::find($request->assignment);

        $bill = $this->bill($assignment);
        $notes = $bill["notes"];
        $payments = $bill["payments"];
        $price = $bill["price"];
        $place = $bill["place"];
        $logo = $assignment->ticket->place->logo;
        $sss = DB::table('short_urls')->where('destination_url','LIKE',"%".$request->assignment."%")->get();
        $ads = $this->getAds($assignment);
        if($assignment->status != CarStatus::PARKING)
            return view('CarStatus', [
                'assignment' => $assignment,
                'logo' => $logo,
                'notes' => $notes,
                'payments' => $payments,
                'total' => $price,
                'place' => $place,
                'sss' => $sss,
                'ads' => $ads
            ]);

        try{
            $response = PaymentClient::getPaymentLink($price, $assignment->id, $assignment->country_code.$assignment->phone);
            if(isset($response->order)){
                $assignment->payment_ref = $response->order->ref;
                $assignment->save();
            }

            return view('RequestCar', ["total" => $price, "payments" => $payments, "notes" => $notes, "logo" => $logo, "name"=>$assignment->ticket->place->name, "assignment"=>$assignment, "response"=>$response, "ads"=>$ads, 'place'=>$place]);
        }catch(Exception $e){
            return view('RequestCar', ["total" => $price, "payments" => $payments, "notes" => $notes, "logo" => $logo, "name"=>$assignment->ticket->place->name, "assignment"=>$assignment, "response"=>$response, "ads"=>$ads, 'place'=>$place]);
        }

    }

    /**
     * Generate the function comment for the given function body in a markdown code block with the correct language syntax.
     *
     * @param Request $request The request object containing the transaction order.
     * @throws Some_Exception_Class description of exception
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View The view with the car status information.
     */
    public function authorizedPayment(Request $request){
        $assignment = Assignment::find(substr($request->Txnorder,0,36));
        $logo = $assignment->ticket->place->logo;
        $bill = $this->bill($assignment);
        $notes = $bill["notes"];
        $payments = $bill["payments"];
        $price = $bill["price"];
        $place = $bill["place"];
        $ads = $this->getAds($assignment);
        if(PaymentClient::check($assignment->payment_ref) == "Paid"){
            if($assignment->status == CarStatus::PARKING){
                $assignment->status = CarStatus::REQUESTED_PAID;
                $assignment->paid = 1;
            }
        }elseif($assignment->status == CarStatus::PARKING){
            $assignment->status = CarStatus::REQUESTED_NOT_PAID;
        }
        $assignment->requested_at = date("Y-m-d H:i:s", strtotime('now'));
        $assignment->save();
        PusherClient::send(
            'car_'.$assignment->ticket->place->id,
            'new_car',
            $assignment
        );
        $notif_config = config('notification-message-config');
        FirebaseClient::send(
            'car_'.$assignment->ticket->place->id,
            $notif_config['title'],
            $notif_config['body']
        );
        return view('CarStatus', [
            'assignment' => $assignment,
            'logo' => $logo,
            'notes' => $notes,
            'payments' => $payments,
            'total' => $price,
            'place' => $place,
            'ads' => $ads,
        ]);
    }

    /**
     * Cancels a payment.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class Description of exception.
     * @return View The view containing the information about the cancelled payment.
     */
    public function cancelPayment(Request $request){
        $assignment = Assignment::find(substr($request->Txnorder,0,36));
        $logo = $assignment->ticket->place->logo;
        $bill = $this->bill($assignment);
        $notes = $bill["notes"];
        $payments = $bill["payments"];
        $price = $bill["price"];
        $place = $bill["place"];
        $ads = $this->getAds($assignment);
        $response = PaymentClient::getPaymentLink($price, $assignment->id, $assignment->country_code.$assignment->phone);
        $assignment->save();
        return view('RequestCar', [
            "total" => $price,
            "payments" => $payments,
            "notes" => $notes,
            "logo" => $logo,
            "name"=>$assignment->ticket->place->name,
            "assignment"=>$assignment,
            "response"=>$response,
            "ads" => $ads,
            'place' => $place,
        ]);
    }

    /**
     * Retrieves a request with cash.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class A description of the exception that can be thrown.
     * @return View The CarStatus view with the assignment, logo, notes, payments, total, and place.
     */
    public function requestWithCash(Request $request){
        $assignment = Assignment::find(substr($request->assignment_id,0,36));
        $logo = $assignment->ticket->place->logo;
        $bill = $this->bill($assignment);
        $notes = $bill["notes"];
        $payments = $bill["payments"];
        $price = $bill["price"];
        $place = $bill["place"];
        $ads = $this->getAds($assignment);
        if($assignment->status == CarStatus::PARKING){
            $assignment->status = CarStatus::REQUESTED_NOT_PAID;
        }
        $assignment->requested_at = date("Y-m-d H:i:s", strtotime('now'));

        $response = PaymentClient::getPaymentLink($price, $assignment->id, $assignment->country_code.$assignment->phone);
        $assignment->save();

        PusherClient::send(
            'car_'.$assignment->ticket->place->id,
            'new_car',
            $assignment
        );
        $notif_config = config('notification-message-config');
        FirebaseClient::send(
            'car_'.$assignment->ticket->place->id,
            $notif_config['title'],
            $notif_config['body']
        );
        return view('CarStatus', [
            'assignment' => $assignment,
            'logo' => $logo,
            'notes' => $notes,
            'payments' => $payments,
            'total' => $price,
            'place' => $place,
            "response"=>$response,
            'ads' => $ads,
        ]);
    }

    /**
     * Calculates the bill for a given assignment.
     *
     * @param Assignment $assignment The assignment for which the bill is calculated.
     * @return array Returns an array with the following keys:
     *         - "notes": An array of strings representing the notes for the bill.
     *         - "payments": An array of strings representing the payments for the bill.
     *         - "price": The total price of the bill.
     *         - "place": The place associated with the assignment.
     */
    public static function bill($assignment){
        $price = 0;
        $notes = array();
        $payments = array();
        $place = $assignment->ticket->place;
        if(!$assignment->is_guest && $assignment->ticket->vip == 0){
            $weekday = strtolower(date('D', strtotime($assignment->created_at->format('Y-m-d H:i:s'))));
            if(in_array($weekday, ["monday", "tuesday", "wednesday"])){
                if($place->week_days_price != 0)
                    array_push($payments, strval($place->week_days_price));
                else
                    array_push($payments, strval($place->all_week_price));

            }
            elseif(in_array($weekday, ["friday"])){
                if($place->friday_price != 0)
                    array_push($payments, strval($place->friday_price));
                elseif($place->week_days_price != 0)
                    array_push($payments, strval($place->week_days_price));
                else
                    array_push($payments, strval($place->all_week_price));

            }else{
                if($place->weekend_price != 0)
                    array_push($payments, strval($place->weekend_price));
                else
                    array_push($payments, strval($place->all_week_price));

            }
            array_push($notes, "Parking");
            $price += $payments[0];

        }


        if($assignment->carwash){
            $price += $place->carwash_price;
            array_push($payments, strval($place->carwash_price));
            array_push($notes, "Carwash");
        }
        if($assignment->fuel){
            $price += $place->fuel_price;
            array_push($payments, strval($place->fuel_price));
            array_push($notes, "Fuel");
        }
        if($assignment->extra_service >= 0){
            $price += $assignment->extra_service;
            array_push($payments, strval($assignment->extra_service));
            array_push($notes, "Extra Service");
        }
        if($assignment->discount >= 0){
            $price -= $assignment->discount;
            array_push($notes, "Discount");
            array_push($payments, strval($assignment->discount));

        }

        if($place->tax >= 0){
            $tax = ($place->tax * $price) / 100;
            $price += $tax;
            array_push($payments, strval($tax));
            array_push($notes,"Tax");
        }
        return
            [
                "notes" => $notes,
                "payments" => $payments,
                "price" => $price,
                "place" => $place,
            ];
    }

    /**
     * Apply a promotion to an assignment.
     *
     * @param Request $request The HTTP request object.
     *
     * @throws Some_Exception_Class If the assignment is not found or the car is not found.
     *
     * @return Response The HTTP response with the promotion value and assignment discount.
     */
    public function applyPromotion(Request $request){
        $assignment = Assignment::find($request->assignmentId);
        if($assignment){
            if(!$assignment->applied_promotion){
                $promotion = Promotion::where('place_uuid', $assignment->ticket->place->id)
                ->where('text', $request->promotionCode)
                ->where('exp', '>', date('Y-m-d', strtotime('now')))
                ->where(function (Builder $query) {
                    $query->where('usage_count', '>', 0)
                          ->orWhere('usage_count', '=', null);
                })
                ->first();
            if($promotion){
                DB::beginTransaction();
                $assignment->discount += intval($promotion->value);
                $assignment->applied_promotion = 1;
                $assignment->promotion_uuid = $promotion->id;
                $assignment->save();
                if($promotion->usage_count != null)
                    $promotion->usage_count = $promotion->usage_count -1 ;
                $promotion->save();
                $bill = $this->bill($assignment);
                $price = $bill["price"];
                $response = PaymentClient::getPaymentLink($price, $assignment->id, $assignment->country_code.$assignment->phone);
                DB::commit();
                return response()->standard("", [
                    "value" => $promotion->value,
                    "discount" => $assignment->discount,
                    "tler_response" => $response,
                ]);
            }else{
                return response()->standard("promotion not found!", null);
            }

            }else{
                return response()->standard("promotion applied before!", null);
            }
        }else{
            return response()->error(401, "car not found!" ,[]);
        }
    }

    public function webtrack(Request $request){
        $validator = Validator::make($request->all(), [
            'contact_point' => ['required', 'string', 'max:50'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());
        $data = $validator->validated();
        $messageTemplates = config('app-messages');
        $pattern = "/^[\+]{0,1}\d+$/";
        try{
            preg_match($pattern, $data['contact_point'], $match);
            if(count($match)>0){
                $phone = preg_replace("/^[\+]/", "", $data['contact_point']);
                $phone = preg_replace("/^0+/", "", $phone);
                foreach(Assignment::where('phone', 'like', substr($phone, -10))->where('status', '!=', CarStatus::RELEASED)->orderBy('created_at', 'desc')->get() as $assignment){
                    if($assignment->country_code.$assignment->phone == "+".$phone){
                        $url = DB::table('short_urls')->where('destination_url','like','%'.$assignment->id.'%')->first();
                        if($assignment->send_sms == 1){
                            $message = str_replace(
                                ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                                [$assignment->ticket->label, $assignment->ticket->place->name, $url->default_short_url, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                                $messageTemplates["sms"]
                            );
                            $smsResult = SMSClient::send($assignment->country_code . $assignment->phone, $message);
                            try{
                                $assignment->sms_id = $smsResult->data->id;
                                $assignment->save();
                            }catch(Exception $e){
                                return response()->error(401,$e->getMessage(),"There was a problem sending the SMS. Please try again!");
                            }

                        }
                        else{
                            $message = str_replace(
                                ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                                [$assignment->ticket->label, $assignment->ticket->place->name, $url->default_short_url, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                                $messageTemplates["whatsapp"]
                            );
                            WhatsappClient::send(substr($assignment->country_code . $assignment->phone, 1, strlen($assignment->country_code . $assignment->phone)), $message);
                        }
                        return response()->standard(404,"[Phone not found!",[]);
                    }
                }
                return response()->standard($phone." not found", null);
            }else{
                $assignment = Assignment::where('email', '=', $data['contact_point'])->orderBy('created_at', 'desc')->first();
                if($assignment){
                    $url = DB::table('short_urls')->where('destination_url','like','%'.$assignment->id.'%')->first();
                    $message = str_replace(
                        ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                        [$assignment->ticket->label, $assignment->ticket->place->name, $url->default_short_url, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                        $messageTemplates["email"]
                    );
                    EmailClient::send($data['contact_point'], $assignment->firstname." ".$assignment->lastname, $assignment->place.":Valet Ticket", $message);
                    return response()->standard("Link was sent to your email.", null);
                }else{
                    return response()->error(404,"Email not found!",[]);
                }
            }
        }catch(Exception $e){
            return response()->error(401,$e->getMessage(),"A problem has occurred. Please try again!");
        }

    }


    public function userInfo(Request $request){
        $assignment = Assignment::find($request->assignment);
        return view('UserInfo', [
            'assignment' => $assignment,
            'place' => $assignment->ticket->place,
            'payments' => array(),
            'total' => 100,
        ]);
    }

    public function profile(Request $request){
        $assignment = Assignment::find($request->assignment_id);
        $ticket = $assignment->ticket;
        if($request->firstname != ""){
            $assignment->firstname = $request->firstname;
        }
        if($request->lastname != ""){
            $assignment->lastname = $request->lastname;
        }
        if($request->phone != ""){
            $assignment->country_code = "+".$request->phonecountrycode;
            $assignment->phone = str_replace("+".$request->phonecountrycode, "", $request->phone);
            $assignment->send_sms = 1;
        }
        if($request->whatsapp != ""){
            $assignment->country_code = "+".$request->phonecountrycode;
            $assignment->phone = str_replace("+".$request->phonecountrycode, "", $request->whatsapp);
            $assignment->send_sms = 0;
        }
        if($request->email != ""){
            $assignment->email = $request->email;
        }
        $assignment->save();

        // $builder = new \AshAllenDesign\ShortURL\Classes\Builder();
        $signed_url = URL::signedRoute('requestCar', ['assignment' => $assignment->id], absolute: false);

        $url = DB::table('short_urls')->where('destination_url','like','%'.$assignment->id.'%')->update(['destination_url' => env('APP_URL').$signed_url]);
        $shortURL = DB::table('short_urls')->where('destination_url',env('APP_URL').$signed_url)->first()->default_short_url;
        $messageTemplates = config('app-messages');
        if($request->phone != ""){
            $message = str_replace(
                ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                $messageTemplates["sms"]
            );
            $smsResult = SMSClient::send($assignment->country_code . $assignment->phone, $message);
            $assignment->sms_id = $smsResult->data->id;
            $assignment->save();
        }
        elseif($request->whatsapp != ""){
            $message = str_replace(
                ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                $messageTemplates["whatsapp"]
            );
        //     return response()->standard("", ["phone"=> substr($assignment->country_code . $assignment->phone, 1, strlen($assignment->country_code . $assignment->phone)),
        // "message"=> $message]);
            WhatsappClient::send(substr($assignment->country_code . $assignment->phone, 1, strlen($assignment->country_code . $assignment->phone)), $message);
            $assignment->sms_id =null;
            $assignment->save();
        }
        if($request->email != ""){
            $message = str_replace(
                ['{TICKET_LABEL}', '{HOTEL_NAME}', '{REQUEST_URL}', '{VALLET_COMPANY}', '{FIRST_NAME}', '{LAST_NAME}', '{CAR_PLATE}', '{CAR_MAKE}', '{CAR_COLOR}'],
                [$ticket->label, $ticket->place->name, $shortURL, env('APP_NAME'), $assignment->firstname, $assignment->lastname, $assignment->car_plate, $assignment->car_make, $assignment->car_color],
                $messageTemplates["email"]
            );
            EmailClient::send($request->email, $ticket->place->name, $assignment->place.":Valet Ticket", $message);
        }
        return redirect(env('APP_URL').$signed_url);

    }
}
