<?php

namespace App\Http\Controllers;

use App\Models\Place;
use App\Models\User;
use App\Models\Ticket;
use App\Models\Ads;
use App\Models\Assignment;
use App\Models\Promotion;
use App\Models\CarDamage;
use Illuminate\Http\Request;
use App\Rules\ArrayValuesInList;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Hash;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Broadcasting\BroadcastException;
use Twilio\Rest\Client;
use Illuminate\Database\Eloquent\Builder;
use Exception;
use App\Helpers\SMSClient;
use App\Helpers\PaymentClient;
use App\Helpers\EmailClient;
use App\Helpers\PusherClient;
use App\Enums\CarStatus;
use AshAllenDesign\ShortURL\Facades\ShortURL;
use Illuminate\Support\Carbon;

class DamageController extends Controller
{
    /**
     * Add damage media.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function addDamageMedia(Request $request){
        $validator = Validator::make($request->all(), [
            'assignment_uuid' => ['required', 'string', 'size:36', 'exists:assignments,id'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());
            $data = $validator->validated();
        if(auth()->user()->id != Assignment::find($data['assignment_uuid'])->user_uuid)
            return response()->unauthorized();

        if ($request->hasFile('media')) {
            try{
                $media = $request->file('media');
                $ext = $media->extension();
                if($ext == 'jpg' || $ext == 'jpeg' || $ext == 'png' || $ext == 'mp4'){
                    $media_name = $media->store("public/car_media");
                    $user = auth()->user();
                    DB::beginTransaction();
                    $carDamage = CarDamage::create([
                        "path" => Storage::url($media_name),
                        "assignment_uuid" => $data['assignment_uuid'],
                        "type" => $ext,
                    ]);
                    DB::commit();
                    return response()->standard("logo was added successfully.", [
                        "carDamage"=> $carDamage
                    ]);
                }else{

                    return response()->validation(["media" => ["media format not supported"]]);
                }
            }catch(QueryException $e){
                DB::rollBack();
                return response()->error(1001, "SQL Error", $e);
            }catch(Exception $e){
                DB::rollBack();
                return response()->error(1002, "Some Error accured!", $e->getMessage());
            }
        }else{
            return response()->validation(["media" => ["media file is required"]]);
        }
    }

    /**
     * Get damage medias.
     *
     * @param string $assignment_uuid The assignment UUID.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function getDamageMedias(string $assignment_uuid){
        $validator = Validator::make(["assignment_uuid" => $assignment_uuid], [
            'assignment_uuid' => ['required', 'string', 'size:36', 'exists:assignments,id'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());
        $data = $validator->validated();
        $assignment = Assignment::find($data['assignment_uuid']);
        if(auth()->payload()->get('default_place') == $assignment->ticket->place->id){
            return response()->standard("", ["damages" => $assignment->carDamages]);
        }else{
            return response()->unauthorized();
        }
    }

    /**
     * Delete a damage media.
     *
     * @param string $damage_uuid The damage UUID.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function deleteDamageMedia(string $damage_uuid){
        $validator = Validator::make(["damage_uuid" => $damage_uuid], [
            'damage_uuid' => ['required', 'string', 'size:36', 'exists:car_damages,id'],
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());
        $data = $validator->validated();
        $damage = CarDamage::find($data['damage_uuid']);
        if(auth()->payload()->get('default_place') == $damage->assignment->ticket->place->id){
            File::delete($damage->path);
            $damage->delete();
            return response()->standard("damage delete successfully", []);
        }else{
            return response()->unauthorized();
        }
    }
}
