<?php

namespace App\Models;

use App\Helpers\TimeModification;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Relations\BelongsTo;


class CarDamage extends Model
{
    use HasUuids, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'path',
        'type',
        'assignment_uuid'
    ];

    protected $appends = [
                            'is_image',
                            'video',
                            'image',
                        ];


    protected function createdAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->assignment->ticket->place->time_zone)
        );
    }

    protected function updatedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->assignment->ticket->place->time_zone)
        );
    }

    protected function deletedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->assignment->ticket->place->time_zone)
        );
    }

    /**
     * Determines if the attribute is an image.
     *
     * @return boolean
     */
    public function getIsImageAttribute()
    {
        return ($this->type != 'mp4');
    }
    /**
     * Get the video attribute.
     *
     * @return string|null The path of the video without the 'storage/' prefix.
     */
    public function getVideoAttribute()
    {
        if($this->type != 'mp4'){
            return null;
        }
        return substr($this->path, strlen('storage/'));
    }
    /**
     * Get the image attribute.
     *
     * @return string|null The image attribute or null if the type is 'mp4'.
     */
    public function getImageAttribute()
    {
        if($this->type == 'mp4'){
            return null;
        }
        return substr($this->path, strlen('storage/'));
    }
    /**
     * Get the assignment that owns the CarDamage.
     */
    public function assignment(): BelongsTo
    {
        return $this->belongsTo(Assignment::class, 'assignment_uuid')->withTrashed();
    }

}
