<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Laravel\Sanctum\HasApiTokens;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\Pivot;
use App\Models\User;
use App\Models\Place;
use App\Enums\RoleType;
use Illuminate\Database\Eloquent\Casts\AsEnumCollection;
use Illuminate\Database\Eloquent\Casts\Attribute;


class Role extends Pivot
{
    use HasUuids, SoftDeletes;
    protected $table = 'roles';
    /**
     * Indicates if the IDs are auto-incrementing.
     *
     * @var bool
     */
    public $incrementing = true;
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'place_uuid',
        'user_uuid',
        'permissions',
    ];

    protected $casts = [
        'permissions' => AsEnumCollection::class.':'.RoleType::class
    ];

    /**
     * The roles that belong to the user.
     */
    public function place(): BelongsTo
    {
        return $this->belongsTo(Place::class, 'place_uuid')->withTrashed();
    }
    /**
     * The roles that belong to the user.
     */
    public function user(): BelongsTo
    {
        return $this->belongsTo(User::class, 'user_uuid')->withTrashed();
    }
}
