<?php

namespace App\Models;

use App\Helpers\TimeModification;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;


class Ticket extends Model
{
    use HasUuids, SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'label',
        'status',
        'vip',
        'place_uuid'
    ];

    protected function createdAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->place->time_zone)
        );
    }

    protected function updatedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->place->time_zone)
        );
    }

    protected function deletedAt(): Attribute
    {
        return Attribute::make(
            get: fn ($time) => TimeModification::setTimeZone($time, $this->place->time_zone)
        );
    }

    /**
     * Get the place that owns the ticket.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function place(): BelongsTo
    {
        return $this->belongsTo(Place::class, 'place_uuid')->withTrashed();
    }
    /**
     * Get all of the assignments for the ticket.
     */
    public function assignments(): HasMany
    {
        return $this->hasMany(Assignment::class, 'ticket_uuid');
    }
}
