<?php

namespace App\Nova\Actions;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Collection;
use Laravel\Nova\Actions\Action;
use Laravel\Nova\Fields\ActionFields;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Fields\BelongsTo;
use App\Nova\User;
use Lednerb\ActionButtonSelector\ShowAsButton;
use App\Helpers\FirebaseClient;
use App\Helpers\PusherClient;
use App\Models\User as ModelsUser;
use Illuminate\Support\Facades\DB;
use Laravel\Nova\Fields\Select;
use Laravel\Nova\Fields\Text;

class AssignTo extends Action
{
    use InteractsWithQueue, Queueable;
    use ShowAsButton;
    public function __construct($model){
        $this->model = $model;
    }

    /**
     * Perform the action on the given models.
     *
     * @param  \Laravel\Nova\Fields\ActionFields  $fields
     * @param  \Illuminate\Support\Collection  $models
     * @return mixed
     */
    public function handle(ActionFields $fields, Collection $models)
    {
        foreach ($models as $m) {
            if($m->assigned_to == null){
                $m->assigned_to = $fields['assignedTo'];
                $m->assigned_at = date("Y-m-d H:i:s", strtotime('now'));
                $m->save();
                PusherClient::send(
                    'car_'.$m->ticket->place->id,
                    'new_car',
                    //$model
                    // \App\Models\User::where('username','=',$fields->assignTo)->get()
                    $fields['assignedTo']
                );
                $notif_config = config('notification-message-config');
                FirebaseClient::send(
                    'car_'.$m->ticket->place->id,
                    $notif_config['title'],
                    $notif_config['body']
                );
                return Action::message("Car Assigned Successfully!");
            }else{
                return Action::danger('This car assigned to another one!');
            }  
        }
    }

    /**
     * Get the fields available on the action.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {

        // return [
        //     BelongsTo::make('Assigned To', 'assignedTo', User::class)
        // ];

        $inputs = [];
        if($this->model->id != null){
            $assignmentId = $this->model->id;
            $users = DB::table('roles')
            ->whereJsonContains('permissions', 'VALET')
            ->join('users', 'users.id', '=', 'roles.user_uuid')
            ->where('roles.place_uuid', '=', \App\Models\Assignment::find($assignmentId)->ticket->place->id)
            ->pluck('users.username', 'users.id')
            ->toArray();

            $inputs[] = 
            // Text::make('subject')->default("bbb");
                Select::make('Assigned To', 'assignedTo')
                    ->options($users)
                    ->searchable()
            ;
        }else{
            $inputs[] = 
            // Text::make('subject')->default("aaa");
                Select::make('Assigned To', 'assignedTo')
                    ->options([])
                    ->searchable()
            ;
        }
        return $inputs;
    }
}
        
