<?php

namespace App\Nova\Actions;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Collection;
use Laravel\Nova\Actions\Action;
use Laravel\Nova\Fields\ActionFields;
use Laravel\Nova\Http\Requests\NovaRequest;
use Barryvdh\DomPDF\Facade\Pdf;
use Illuminate\Support\Facades\DB;
use Laravel\Nova\Fields\Date;
use Lednerb\ActionButtonSelector\ShowAsButton;

class CashRecievedAction extends Action
{
    use InteractsWithQueue, Queueable;
    use ShowAsButton;
    public $name = "Cash Report";

    /**
     * Perform the action on the given models.
     *
     * @param  \Laravel\Nova\Fields\ActionFields  $fields
     * @param  \Illuminate\Support\Collection  $models
     * @return mixed
     */
    public function handle(ActionFields $fields, Collection $models)
    {
        // $pdf = \App::make('dompdf.wrapper');
        // $pdf->loadHTML('<h1>Test</h1>');
        // return ;
        $fromDate = $fields["from"];
        $toDate = $fields["to"];
        foreach($models as $model){
            $pdf = \App::make('dompdf.wrapper');
            $assignments = DB::table('assignments')
                ->join('tickets', 'assignments.ticket_uuid', '=', 'tickets.id')
                ->join('users', 'cash_reciever', '=', 'users.id')
                ->where('tickets.place_uuid', '=', $model->id)
                ->whereBetween('assignments.created_at', [$fromDate, $toDate])
                ->selectRaw('sum(charge) as cash, users.username as username, users.id as user_id, users.firstname as firstname, users.lastname as lastname')
                ->groupBy('user_id')->get();
            $sumCash = 0; 
            foreach($assignments as $assignment){
                $sumCash += $assignment->cash;
            }
            $pdf->loadView('nova-pdf/template', [
                "model" => $assignments,
                "sumCash" => $sumCash,
                'place'=>$model,
                'from_date'=>(new \DateTime($fromDate))->format('d M Y'),
                'to_date'=>(new \DateTime($toDate))->format('d M Y'),
                
                ])
            ->setPaper('a4');
            $id = uniqid();
            // $path = '/tmp/test'. $id .'.pdf';
            $pdf->save('storage/invoices/invoice'. $id . '.pdf');
            return Action::openInNewTab(url('storage/invoices/'.'invoice'. $id . '.pdf'));//, $model->name.'_'.$fromDate.'_'.$toDate.'_invoice.pdf');
        }
    }

    /**
     * Get the fields available on the action.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        return [
            Date::make('From Date', 'from'),
            Date::make('To Date', 'to'),
        ];
    }
}
