<?php

namespace App\Nova\Actions;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Collection;
use Laravel\Nova\Actions\Action;
use Laravel\Nova\Fields\ActionFields;
use Laravel\Nova\Http\Requests\NovaRequest;
use App\Enums\CarStatus;
use App\Helpers\PusherClient;
use Laravel\Nova\Fields\Number;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Currency;
use App\Nova\Assignment;
use Laravel\Nova\Fields\KeyValue;

class RecieveCash extends Action
{
    use InteractsWithQueue, Queueable;

    public function __construct($model=null){
        $this->model = $model;
    }
    /**
     * Perform the action on the given models.
     *
     * @param  \Laravel\Nova\Fields\ActionFields  $fields
     * @param  \Illuminate\Support\Collection  $models
     * @return mixed
     */
    public function handle(ActionFields $fields, Collection $models)
    {

        foreach ($models as $model) {
            if($model->paid == 1)
                return Action::danger('Already paid!');
            $model->cash_reciever = auth()->guard('web')->user()->id;
            if($model->status == CarStatus::REQUESTED_NOT_PAID){
                $model->status = CarStatus::REQUESTED_PAID;
                $model->paid = 1;
            }
            if($model->status == CarStatus::READY_NOT_PAID){
                $model->status = CarStatus::READY_PAID;
                $model->paid = 1;
            }
            $model->save();
            PusherClient::send(
                'car_'.$model->ticket->place->id,
                'new_car',
                $model
            );
            return Action::message('The cash payment was successfully recieved!');
        }
    }

    /**
     * Get the fields available on the action.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {

        if($this->model != null){
            $fields = array();
            for($i=0; $i<count($this->model->price['notes']); $i++){
                array_push($fields, Currency::make($this->model->price['notes'][$i])->default($this->model->price['payments'][$i])->readonly()->textAlign('right')->currency('AED'));
            }
            array_push($fields, Currency::make('Total')->default($this->model->price['price'])->readonly()->textAlign('right')->currency('AED'));
            return $fields;
        }else{
            return [];
        }

    }
}


// searchIndex -- toCount -- perPage -- isInlineCreateRequest -- isCreateOrAttachRequest -- isUpdateOrUpdateAttachedRequest -- isResourceIndexRequest -- isResourceDetailRequest -- isResourcePreviewRequest -- isResourcePeekingRequest -- isLensRequest -- isActionRequest -- isFormRequest -- isPresentationRequest -- createFromBase -- validationData -- safe -- validated -- messages -- attributes -- setValidator -- setRedirector -- setContainer -- capture -- instance -- method -- root -- url -- fullUrl -- fullUrlWithQuery -- fullUrlWithoutQuery -- path -- decodedPath -- segment -- segments -- is -- routeIs -- fullUrlIs -- host -- httpHost -- schemeAndHttpHost -- ajax -- pjax -- prefetch -- secure -- ip -- ips -- userAgent -- merge -- mergeIfMissing -- replace -- get -- json -- createFrom -- duplicate -- hasSession -- getSession -- session -- setLaravelSession -- setRequestLocale -- setDefaultRequestLocale -- user -- route -- fingerprint -- setJson -- getUserResolver -- setUserResolver -- getRouteResolver -- setRouteResolver -- toArray -- offsetExists -- offsetGet -- offsetSet -- offsetUnset -- __isset -- __get -- __construct -- initialize -- createFromGlobals -- create -- setFactory -- __clone -- __toString -- overrideGlobals -- setTrustedProxies -- getTrustedProxies -- getTrustedHeaderSet -- setTrustedHosts -- getTrustedHosts -- normalizeQueryString -- enableHttpMethodParameterOverride -- getHttpMethodParameterOverride -- hasPreviousSession -- setSession -- setSessionFactory -- getClientIps -- getClientIp -- getScriptName -- getPathInfo -- getBasePath -- getBaseUrl -- getScheme -- getPort -- getUser -- getPassword -- getUserInfo -- getHttpHost -- getRequestUri -- getSchemeAndHttpHost -- getUri -- getUriForPath -- getRelativeUriForPath -- getQueryString -- isSecure -- getHost -- setMethod -- getMethod -- getRealMethod -- getMimeType -- getMimeTypes -- getFormat -- setFormat -- getRequestFormat -- setRequestFormat -- getContentType -- getContentTypeFormat -- setDefaultLocale -- getDefaultLocale -- setLocale -- getLocale -- isMethod -- isMethodSafe -- isMethodIdempotent -- isMethodCacheable -- getProtocolVersion -- getContent -- getPayload -- getETags -- isNoCache -- getPreferredFormat -- getPreferredLanguage -- getLanguages -- getCharsets -- getEncodings -- getAcceptableContentTypes -- isXmlHttpRequest -- preferSafeContent -- isFromTrustedProxy -- filterPrecognitiveRules -- isAttemptingPrecognition -- isPrecognitive -- isJson -- expectsJson -- wantsJson -- accepts -- prefers -- acceptsAnyContentType -- acceptsJson -- acceptsHtml -- matchesType -- format -- old -- flash -- flashOnly -- flashExcept -- flush -- server -- hasHeader -- header -- bearerToken -- exists -- has -- hasAny -- whenHas -- filled -- isNotFilled -- anyFilled -- whenFilled -- missing -- whenMissing -- keys -- all -- input -- str -- string -- boolean -- integer -- float -- date -- enum -- collect -- only -- except -- query -- post -- hasCookie -- cookie -- allFiles -- hasFile -- file -- dd -- dump -- macro -- mixin -- hasMacro -- flushMacros -- __callStatic -- __call -- validateResolved -- authorize -- rules -- resourceSoftDeletes -- resource -- newResource -- findResourceOrFail -- findResource -- findModelOrFail -- findModel -- findModelQuery -- newResourceWith -- newQuery -- newQueryWithoutScopes -- model -- findParentResource -- findParentResourceOrFail -- findParentModel -- findParentModelOrFail -- findRelatedResource -- findRelatedResourceOrFail -- findRelatedModel -- findRelatedModelOrFail -- pivotName -- relatedResource -- newRelatedResource -- viaResource -- newViaResource -- viaRelationship -- viaManyToMany -- allResourcesSelected -- selectedResourceIds -- selectedResources -- buildCountQuery -- toQuery -- orderings -- filters
