<?php

namespace App\Nova\Filters;

use Laravel\Nova\Filters\Filter;
use Laravel\Nova\Http\Requests\NovaRequest;
use App\Models\Place;
use Carbon\Carbon;

class PlaceFilter extends Filter
{
    /**
     * Apply the filter to the given query.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @param  \Illuminate\Database\Eloquent\Builder  $query
     * @param  mixed  $value
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function apply(NovaRequest $request, $query, $value)
    {
        $tickets_id = Place::where('name', $value)->first()->tickets->pluck('id')->toArray();
        return $query->whereIn('ticket_uuid', $tickets_id);
    }

    /**
     * Get the filter's available options.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function options(NovaRequest $request)
    {
        if ($request->user()->superadmin) {
            return Place::all()->pluck('name');
        } else {
            // return auth()->guard('web')->user()->id;
        return Place::join('roles', 'roles.place_uuid', 'places.id')
        ->where('roles.user_uuid', auth()->guard('web')->user()->id)
        ->pluck('places.name');
        }
    }
}