<?php

namespace App\Nova\Lenses;

use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Http\Requests\LensRequest;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Lenses\Lens;
use Laravel\Nova\Nova;
use App\Enums\CarStatus;
use Laravel\Nova\Fields\Number;
use Laravel\Nova\Fields\Date;
use Laravel\Nova\Fields\DateTime;
use Laravel\Nova\Fields\Boolean;
use Laravel\Nova\Fields\HasOne;
use Laravel\Nova\Fields\HasMany;
use Laravel\Nova\Fields\BelongsTo;
use App\Nova\Ticket;
use Laravel\Nova\Fields\Email;
use SimpleSquid\Nova\Fields\Enum\Enum;
use Pavloniym\ActionButtons\ActionButton;
use Pavloniym\ActionButtons\ActionButtons;
use Laravel\Nova\Panel;
use App\Nova\Actions\AssignTo;
use App\Nova\Actions\RecieveCash;
use App\Nova\Assignment;
use Illuminate\Support\Facades\Log;

class RequestedCars extends Lens
{
    public static $model = \App\Models\Assignment::class;
    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [];

    /**
     * Get the query builder / paginator for the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\LensRequest  $request
     * @param  \Illuminate\Database\Eloquent\Builder  $query
     * @return mixed
     */
    public static $sort = [
        'assignments.updated_at' => 'desc',
    ];
    public static function query(LensRequest $request, $query)
    {
        if(auth()->guard('web')->user()->superadmin){
            return $request->withOrdering($request->withFilters(
                $query->select('assignments.*')
                    ->join('tickets', 'ticket_uuid', 'tickets.id')
                    ->whereIn('assignments.status', [CarStatus::REQUESTED_PAID, CarStatus::REQUESTED_NOT_PAID])
                    ->whereNull('assigned_to')
                    ->orderBy(key(static::$sort), reset(static::$sort))
            ));
        }
        return $request->withOrdering($request->withFilters(
                $query->select('assignments.*')
                    ->join('tickets', 'ticket_uuid', 'tickets.id')
                    ->whereIn('assignments.status', [CarStatus::REQUESTED_PAID, CarStatus::REQUESTED_NOT_PAID])
                    ->whereNull('assigned_to')
                    ->whereIn('tickets.place_uuid', auth()->guard('web')->user()->places->pluck('id')->toArray())
                    ->orderBy(key(static::$sort), reset(static::$sort))
        ));
    }

    /**
     * Get the fields available to the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        // Log::info(serialize($this->resource));
        $fields =  [
            ID::make('uuid')->hidden()->sortable(),
            Text::make('Place')->hideWhenUpdating(),
            Belongsto::make('Ticket', 'ticket', Ticket::class)->readOnly()->searchable(),
            Number::make('Phone', 'phone')->hideFromIndex(),
            Email::make('Email', 'email')->hideFromIndex(),
            Enum::make('Status','status')->hideWhenUpdating()->displayUsing(function($value){
                if($value == 'parking')
                    return 'Check In';
                else
                    return $value;
            }),
            Text::make('Car Plate', 'car_plate')->hideWhenUpdating()->readOnly(),
            Text::make('Car Make', 'car_make')->hideWhenUpdating(),
            Text::make('Level', 'level')->hideWhenUpdating(),
            Text::make('Slot', 'slot')->hideWhenUpdating(),
            Text::make('Hanger', 'hanger')->hideWhenUpdating(),
            DateTime::make('Entrance','created_at')->hideWhenUpdating()->displayUsing(fn ($value) => $value ? $value->format('d M Y,') . ' ' . $value->format('h:i A') : ' '),
            DateTime::make('Last Update','updated_at')->hideWhenUpdating()->displayUsing(fn ($value) => $value ? $value->format('d M Y,') . ' ' . $value->format('h:i A') : ' '),
            Boolean::make("Carwash"),
            Boolean::make("Fuel"),
            Boolean::make("Paid")->hideWhenUpdating(),
            // Text::make('Bill', 'price'),
            (new Panel('Guest', $this->guest())),
            Boolean::make('Damages', 'hasDamage'),
            (new Panel('Valet', $this->userFields())),
            HasMany::make('Damages', 'carDamages', 'App\Nova\CarDamage'),
        ];
        
        if(serialize($this->resource) != "O:8:\"stdClass\":0:{}"){
            Log::info("ASSIGN_TO ACTION: \n".serialize($this->resource));
            $fields[] =  ActionButton::make('Assign To')->icon('<svg xmlns="http://www.w3.org/2000/svg" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor" class="w-6 h-6">
            <path stroke-linecap="round" stroke-linejoin="round" d="M18 7.5v3m0 0v3m0-3h3m-3 0h-3m-2.25-4.125a3.375 3.375 0 1 1-6.75 0 3.375 3.375 0 0 1 6.75 0ZM3 19.235v-.11a6.375 6.375 0 0 1 12.75 0v.109A12.318 12.318 0 0 1 9.374 21c-2.331 0-4.512-.645-6.374-1.766Z" />
            </svg>
            ')->styles([
            'background-color' => 'rgba(0, 0, 0, 0)',
            'shadow' => 'none',
            ])
            ->action(new AssignTo($this->resource),$this->resource->id);
            $fields[] = ActionButton::make('Recieve Cash')->icon('<svg xmlns="http://www.w3.org/2000/svg" class="h-6 w-6" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke="currentColor" stroke-width="1.5">
            <path stroke-linecap="round" stroke-linejoin="round" d="M17 9V7a2 2 0 00-2-2H5a2 2 0 00-2 2v6a2 2 0 002 2h2m2 4h10a2 2 0 002-2v-6a2 2 0 00-2-2H9a2 2 0 00-2 2v6a2 2 0 002 2zm7-5a2 2 0 11-4 0 2 2 0 014 0z" />
            </svg>
            ')->styles([
            'background-color' => 'rgba(0, 0, 0, 0)',
            'shadow' => 'none',
            ])
            ->action((new RecieveCash($this->resource))->confirmText('Customer`s payment has been received. Are you sure?')
            ->confirmButtonText('Paid')
            ->cancelButtonText("Not Paid"), $this->resource->id);
        }
        return $fields;
    }

    protected function guest()
    {
        return [
            Boolean::make("Guest", "is_guest")->filterable(),
            Text::make("Room", "room")->hideFromIndex(),
            Date::make("Check Out", "checkout_date")->hideFromIndex(),
        ];
    }
    /**
     * Get the valet fields for the resource.
     *
     * @return array
     */
    protected function userFields()
    {
        return [
            BelongsTo::make('Transferee', 'user', \App\Nova\User::class)->showOnDetail()->withoutTrashed()->readOnly(),
            BelongsTo::make('Assigned To', 'assignedTo', \App\Nova\User::class)->showOnDetail()->nullable()->withoutTrashed(),
            Text::make('Level', 'level')->showOnDetail()->hideFromIndex()->readOnly(),
            Text::make('Slot',  'slot')->showOnDetail()->hideFromIndex()->readOnly(),
            Text::make('Hanger', 'hanger')->showOnDetail()->hideFromIndex()->readOnly(),
        ];
    }

    /**
     * Get the cards available on the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function cards(NovaRequest $request)
    {
        return [];
    }

    public static $showPollingToggle = true;
    public static $pollingInterval = 30;
    /**
     * Get the filters available for the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function filters(NovaRequest $request)
    {
        return [
            new \App\Nova\Filters\PlaceFilter(),
            new \App\Nova\Filters\FromDateFilter(),
            new \App\Nova\Filters\ToDateFilter(),
        ];
    }

    /**
     * Get the actions available on the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function actions(NovaRequest $request)
    {
        return parent::actions($request);
    }

    /**
     * Get the URI key for the lens.
     *
     * @return string
     */
    public function uriKey()
    {
        return 'requested-cars';
    }
    public static function relatableUsers(NovaRequest $request, $query)
    {
        if(\App\Models\Assignment::find($request->resourceId))
            return $query->whereIn('id', \App\Models\Assignment::find($request->resourceId)->ticket->place->users->pluck('id')->toArray());
        return $query;
    }

    public static function indexQuery(NovaRequest $request, $query)
    {
        if($request->user()->superadmin){
            return $query;
        }
            return $query
                ->join('tickets', 'assignments.ticket_uuid', '=', 'tickets.id')
                ->whereIn('tickets.place_uuid', $request->user()->places->pluck('id')->toArray())
                ->select('assignments.*');

    }
    public function lensesData($lens)
    {
        return [
            'assignments.id' => LensRequest::getCurrentassignmentsId(),
        ];
    }

}