<?php

namespace App\Nova\Widgets;

use DigitalCreative\ChartJsWidget\Charts\LineChartWidget;
use DigitalCreative\NovaDashboard\Filters;
use Illuminate\Support\Collection;
use Laravel\Nova\Http\Requests\NovaRequest;
use Illuminate\Support\Facades\DB;
use App\Nova\Filters\PlaceFilter;
use App\Nova\Filters\ToDateFilter;
use App\Nova\Filters\FromDateFilter;

class CarTrendWidget extends LineChartWidget
{
    /**
     * Retrieves the value based on the given filters.
     *
     * @param Filters $filters The filters to apply.
     * @return array The value retrieved.
     */
    public function configure(NovaRequest $request): void
    {
        /**
         * These set the title and the button on the top-right if there are multiple "tabs" on this view
         */
        $this->title('Parking Trend');
        // $this->buttonTitle('BarChart');
        $this->backgroundColor(dark: '#1e293b', light: '#f50fff');

        $this->padding(top: 30, bottom: 5);

        // $this->tooltip([]); // https://www.chartjs.org/docs/latest/configuration/tooltip.html#tooltip
        // $this->scales([]);  // https://www.chartjs.org/docs/latest/axes/#axes
        // $this->legend([]);  // https://www.chartjs.org/docs/latest/configuration/legend.html#legend
        // $this->elements();  // https://www.chartjs.org/docs/latest/configuration/elements.html#elements
    }

    public function value(Filters $filters): array
    {
        $fromValue = $filters->getFilterValue(FromDateFilter::class);
        $toValue = $filters->getFilterValue(ToDateFilter::class);
        $placeValue = $filters->getFilterValue(PlaceFilter::class);
        if(!isset($fromValue)){
            $fromValue = date('Y-m-1',strtotime('now'));
        }
        if(!isset($toValue)){
            $toValue = date('Y-m-t',strtotime('now'));
        }
        if(!isset($placeValue)){
            $places = \App\Models\Place::all();
            $datasets = [];
            $labels = [];
            foreach(DB::table('assignments')->selectRaw('DATE(assignments.created_at) as date')->distinct("date")
                        ->where('assignments.created_at', '>=', $fromValue)
                        ->where('assignments.created_at', '<=', $toValue)
                        ->get() as $date)
                array_push($labels, $date->date);
            foreach($places as $place){
                $results = DB::table('assignments')
                ->where('assignments.created_at', '>=', $fromValue)
                ->where('assignments.created_at', '<=', $toValue)
                ->join('tickets', 'assignments.ticket_uuid', "=", "tickets.id")
                ->join('places', 'tickets.place_uuid', "=", "places.id")
                ->where('places.id', $place->id)
                ->selectRaw('count(assignments.id) as count, DATE(assignments.created_at) as date, places.name, places.id' )
                ->groupBy('places.id')
                ->groupBy('date')->get();
                $data = array_fill(0, count($labels), 0);
                foreach($results as $result){
                    $data[array_search($result->date, $labels)]=$result->count;
                }

                array_push($datasets,[
                    'label'=> $place->name,
                    'data' => $data
                ]);
            }

        }else{
            $places = \App\Models\Place::where('name',$placeValue)->get();
            $datasets = [];
            $labels = [];
            foreach(DB::table('assignments')->selectRaw('DATE(assignments.created_at) as date')->distinct("date")
                        ->where('assignments.created_at', '>=', $fromValue)
                        ->where('assignments.created_at', '<=', $toValue)
                        ->get() as $date)
                array_push($labels, $date->date);
            foreach($places as $place){
                $results = DB::table('assignments')
                ->where('assignments.created_at', '>=', $fromValue)
                ->where('assignments.created_at', '<=', $toValue)
                ->join('tickets', 'assignments.ticket_uuid', "=", "tickets.id")
                ->join('places', 'tickets.place_uuid', "=", "places.id")
                ->where('places.id', $place->id)
                ->selectRaw('count(assignments.id) as count, DATE(assignments.created_at) as date, places.name, places.id' )
                
                ->groupBy('places.id')
                ->groupBy('date')->get();
                $data = array_fill(0, count($labels), 0);
                foreach($results as $result){
                    $data[array_search($result->date, $labels)]=$result->count;
                }

                array_push($datasets,[
                    'label'=> $place->name,
                    'data' => $data
                ]);
            }
        }
        return [
            'labels' => $labels,
            'datasets' => $datasets,
        ];
    }
}
