<?php

namespace App\Providers;

use App\Models\Role;

use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\Response;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     */
    public function register(): void
    {
        // Role::observe(RoleObserver::class);
    }

    /**
     * Bootstrap any application services.
     */
    public function boot(): void
    {
        Response::macro('standard', function (string $message, $object) {
            return response()->json([
                "status"   => 200,
                "isSuccess" => true,
                "message"=> $message,
                "data"  => $object,
                "errors" => null
            ], 200);
        });
        Response::macro('error', function (int $code, string $message, $e) {
            return response()->json([
                "isSuccess" => false,
                "status"   => $code,
                "message"=> $message,
                "data" => null,
                "errors"  => $e
            ], 500);
        });

        Response::macro('validation', function ($error) {
            return response()->json([
                "isSuccess" => false,
                "status"   => 400,
                "message"=> "Input validation error!",
                "data" => null,
                "errors"  => $error
            ], 400);
        });
        Response::macro('unauthorized', function () {
            return response()->json([
                "isSuccess" => false,
                "status"   => 401,
                "message"=> "Unauthorized!",
                "data" => null,
                "errors" => null
            ], 401);
        });

        Response::macro('not_found', function () {
            return response()->json([
                "isSuccess" => false,
                "status"   => 404,
                "message"=> "Route not found!",
                "data" => null,
                "error" => null
            ], 404);
        });
    }
}
