<?php

use App\Http\Controllers\AssignmentController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\AuthController;
use App\Http\Controllers\DamageController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\CustomerController;
use App\Http\Middleware\Authenticate;
use Kreait\Firebase\Messaging\CloudMessage;
use Kreait\Firebase\Messaging\Notification;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "api" middleware group. Make something great!
|
*/

Route::controller(AuthController::class)->group(function () {

    // Endpoint for user login
    Route::post('login', 'login');

    // Endpoint for unauthorized access
    Route::get('login', 'notAuthorized');

    // Endpoint for user registration
    Route::post('register', 'register');

    // Endpoint for user logout
    Route::post('logout', 'logout');

    // Endpoint for token refresh
    Route::post('refresh', 'refresh');

    // Endpoint for forget password
    Route::post('forgetPassword', 'forgetPassword');

    // Endpoint for reset password
    Route::post('resetPassword', 'resetPassword');

    // Endpoint for login with verification code
    Route::post('loginWithCode', 'loginWithCode');

    // Endpoint for sending verification code
    Route::post('sendVerificationCode', 'sendVerificationCode');

});


// User routs
Route::controller(UserController::class)->middleware('auth:api')->group(function(){

    // Endpoint for getting user places
    Route::get('getPlaces', 'getUserPlaces');

    // Endpoint for satting defautl place for user
    Route::post('setDefaultPlace', 'setDefaultPlace');

});


// Damage routs
Route::controller(DamageController::class)->middleware('auth:api')->group(function(){

    // Endpoint for adding car damage media
    Route::post('carDamage', 'addDamageMedia');

    // Endpoint for getting car damage media
    Route::get('carDamages/{assignment_uuid}', 'getDamageMedias');

    // Endpoint for deleting car damage media
    Route::delete('carDamage/{damage_id}', 'deleteDamageMedia');
});

// Assignment routs
Route::controller(AssignmentController::class)->middleware('auth:api')->group(function(){

    // Endpoint for adding QR assignment
    Route::post('QRassignment', 'addQRAssignment');
    
    // Endpoint for adding assignment
    Route::post('assignment', 'addAssignment');

    // Endpoint for getting assignment
    Route::get('assignment/{assignment_id}', 'getAssignment');

    // Endpoint for getting assignment list
    Route::get('assignmentList/{page?}', 'getAssignmentList');

    // Endpoint for editting assignment
    Route::put('assignment', 'editAssignment');

    // Endpoint for deleting assignment
    Route::delete('assignment', 'deleteAssignment');

    // Endpoint for checking ticket
    Route::get('checkTicket/{label}', 'checkticket');

    // Endpoint for accepting request
    Route::post('acceptRequest', 'acceptRequest');

    // Endpoint for releasing assignment
    Route::post('releaseAssignment','releaseAssignment');

    // Endpoint for getting ready the assignment
    Route::post('readyAssignment', 'readyAssignment');

    // Endpoint for selecting car color
    Route::get('selectColor', 'selectColor');

    // Endpoint for selecting car brand
    Route::get('selectCarBrand', 'selectCarBrand');

    // Endpoint for resending message
    Route::post('resendMessage', 'resendMessage');
});


// Endpoint for not found error
Route::fallback(function(){
    return response()->not_found();
});

// Endpoint for sending sms
Route::get("sendSMS", function(){
    return \App\Helpers\SMSClient::checkDelivery("05adf53f-78cc-11ee-88a3-6045bd6aa444");
});


Route::post('/webtrack', [CustomerController::class, 'webtrack']);
Route::get("/firebase", function(){
    $messaging = app('firebase.messaging');
    $topic = 'a-topic';
    $notification = Notification::fromArray([
        'title' => "NEW REQUEST",
        'body' => "SALAM",
        'image' => Null,
    ]);
    $data = [
        'first_key' => 'First Value',
        'second_key' => 'Second Value',
    ];
    $message = CloudMessage::withTarget('topic', $topic)
        ->withNotification($notification) // optional
        ->withData($data) // optional
    ;
    try{

        $result = $messaging->send($message);
        return response()->standard("", $result);
    }catch(Exception $e){
        return response()->error(1001,"",$e->getMessage());
    }
});


 
// Route::get('/migration', function () {
//     $exitCode = Artisan::call('migrate');
//     return response()->standard("",$exitCode);
// });
